"""
Project
FPGA-Imaging-Library

Design
RowsGenerator

Function
Generate rows cache. 
The lowest "color_width" bits of "out_data" is the first row! 

Module
Software simulation.

Version
1.0

Modified
2015-05-17

Copyright (C) 2015  Tianyu Dai (dtysky) <dtysky@outlook.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

Homepage for this project:
	http://fil.dtysky.moe

Sources for this project:
	https://github.com/dtysky/FPGA-Imaging-Library

My e-mail:
	dtysky@outlook.com

My blog:
	http://dtysky.moe
"""

__author__ = 'Tianyu Dai (dtysky)'

from PIL import Image
import os, json
from RowsGenerator import RowsGenerator as RG
from ctypes import *
user32 = windll.LoadLibrary('user32.dll')
MessageBox = lambda x:user32.MessageBoxA(0, x, 'Error', 0) 

FileFormat = ['.jpg', '.bmp']
Conf = json.load(open('../ImageForTest/conf.json', 'r'))['conf']
"""Debug must be True in this mode !"""
Debug = True

def show_error(e):
	MessageBox(e)
	exit(0)

def name_format(root, name, ex, conf):
	return '%s-%s-soft' % (name, conf['width'])

def transform(im, conf):
	return []

def debug(im, conf):
	mode = im.mode
	xsize = im.size[0]
	if mode not in ['L', '1']:
		show_error('Simulations for this module just supports Gray-scale and binary images, check your images !')
	if xsize != 512:
		show_error('Simulations for this module just supports 512xN images, check your images !')
	if conf['width'] not in [3, 5]:
		show_error('''Simulations for this module just supports conf "width" 3 and 5, check your images !''')
	rows = RG(im, conf['width'])
	data_res = 'Showed row1 -> rowN\nBut the lowest color_width-bits of this are the first row !\n\n'
	while not rows.frame_empty():
		row = str(rows.update())
		if mode == '1':
			row = row.replace('255', '1')
		data_res += '%s\n' % row.replace('[', '').replace(']', '').replace(',', '')
	return data_res[:-1]

FileAll = []
for root, dirs, files in os.walk('../ImageForTest'):
    for f in files:
    	name, ex = os.path.splitext(f)
        if ex in FileFormat:
        	FileAll.append((root+'/', name, ex))
for root, name, ex in FileAll:
	im_src = Image.open(root + name + ex)
	for c in Conf:
		if Debug:
			with open('../SimResCheck/%s.dat' \
				% name_format(root, name, ex, c), 'w') as fo:
				fo.write(debug(im_src, c))
			continue
		transform(im_src, c).save('../SimResCheck/%s' % name_format(root, name, ex, c))